#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_access_policies
short_description: Resource module for networks _switch _access _policies
description:
  - Manage operations create, update and delete of the resource networks _switch _access
    _policies. - > Create an access policy for a switch network. If you would like
    to enable Meraki Authentication, set radiusServers to empty array.
  - Delete an access policy for a switch network. - > Update an access policy for
    a switch network. If you would like to enable Meraki Authentication, set radiusServers
    to empty array.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  accessPolicyNumber:
    description: AccessPolicyNumber path parameter. Access policy number.
    type: str
  accessPolicyType:
    description: Access Type of the policy. Automatically 'Hybrid authentication'
      when hostMode is 'Multi-Domain'.
    type: str
  dot1x:
    description: 802.1x Settings.
    suboptions:
      controlDirection:
        description: Supports either 'both' or 'inbound'. Set to 'inbound' to allow
          unauthorized egress on the switchport. Set to 'both' to control both traffic
          directions with authorization. Defaults to 'both'.
        type: str
    type: dict
  guestGroupPolicyId:
    description: Group policy Number for guest group policy.
    type: str
  guestPortBouncing:
    description: If enabled, Meraki devices will periodically send access-request
      messages to these RADIUS servers.
    type: bool
  guestSgtId:
    description: Security Group Tag ID for guest group policy.
    type: int
  guestVlanId:
    description: ID for the guest VLAN allow unauthorized devices access to limited
      network resources.
    type: int
  hostMode:
    description: Choose the Host Mode for the access policy.
    type: str
  increaseAccessSpeed:
    description: Enabling this option will make switches execute 802.1X and MAC-bypass
      authentication simultaneously so that clients authenticate faster. Only required
      when accessPolicyType is 'Hybrid Authentication.
    type: bool
  name:
    description: Name of the access policy(max length 255).
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  radius:
    description: Object for RADIUS Settings.
    suboptions:
      authentication:
        description: Object for authentication mode settings.
        suboptions:
          mode:
            description: Authentication mode of the policy ( Open | Closed ).
            type: str
        type: dict
      cache:
        description: Object for RADIUS Cache Settings.
        suboptions:
          enabled:
            description: Enable to cache authorization and authentication responses
              on the RADIUS server.
            type: bool
          timeout:
            description: If RADIUS caching is enabled, this value dictates how long
              the cache will remain in the RADIUS server, in hours, to allow network
              access without authentication.
            type: int
        type: dict
      criticalAuth:
        description: Critical auth settings for when authentication is rejected by
          the RADIUS server.
        suboptions:
          dataGroupPolicyId:
            description: Group policy Number for data VLAN.
            type: str
          dataSgtId:
            description: Security Group Tag ID for data VLAN.
            type: int
          dataVlanId:
            description: VLAN that clients who use data will be placed on when RADIUS
              authentication fails. Will be null if hostMode is Multi-Auth.
            type: int
          suspendPortBounce:
            description: Disable port bounce when RADIUS servers are unreachable.
            type: bool
          voiceGroupPolicyId:
            description: Group policy Number for voice VLAN.
            type: str
          voiceSgtId:
            description: Security Group Tag ID for voice VLAN.
            type: int
          voiceVlanId:
            description: VLAN that clients who use voice will be placed on when RADIUS
              authentication fails. Will be null if hostMode is Multi-Auth.
            type: int
        type: dict
      failedAuthGroupPolicyId:
        description: Group policy Number for failed authentication group policy.
        type: str
      failedAuthSgtId:
        description: Security Group Tag ID for failed authentication group policy.
        type: int
      failedAuthVlanId:
        description: VLAN that clients will be placed on when RADIUS authentication
          fails. Will be null if hostMode is Multi-Auth.
        type: int
      preAuthenticationGroupPolicyId:
        description: Group policy Number for pre-authentication group policy.
        type: str
      reAuthenticationInterval:
        description: Re-authentication period in seconds. Will be null if hostMode
          is Multi-Auth.
        type: int
    type: dict
  radiusAccountingEnabled:
    description: Enable to send start, interim-update and stop messages to a configured
      RADIUS accounting server for tracking connected clients.
    type: bool
  radiusAccountingServers:
    description: List of RADIUS accounting servers to require connecting devices to
      authenticate against before granting network access.
    elements: dict
    suboptions:
      host:
        description: Public IP address of the RADIUS accounting server.
        type: str
      organizationRadiusServerId:
        description: Organization wide RADIUS server ID. If this field is provided,
          the host, port and secret field will be ignored.
        type: str
      port:
        description: UDP port that the RADIUS Accounting server listens on for access
          requests.
        type: int
      secret:
        description: RADIUS client shared secret.
        type: str
    type: list
  radiusCoaSupportEnabled:
    description: Change of authentication for RADIUS re-authentication and disconnection.
    type: bool
  radiusGroupAttribute:
    description: Acceptable values are `""` for None, or `"11"` for Group Policies
      ACL.
    type: str
  radiusServers:
    description: List of RADIUS servers to require connecting devices to authenticate
      against before granting network access.
    elements: dict
    suboptions:
      host:
        description: Public IP address of the RADIUS server.
        type: str
      organizationRadiusServerId:
        description: Organization wide RADIUS server ID. If this field is provided,
          the host, port and secret field will be ignored.
        type: str
      port:
        description: UDP port that the RADIUS server listens on for access requests.
        type: int
      secret:
        description: RADIUS client shared secret.
        type: str
    type: list
  radiusTestingEnabled:
    description: If enabled, Meraki devices will periodically send access-request
      messages to these RADIUS servers.
    type: bool
  urlRedirectWalledGardenEnabled:
    description: Enable to restrict access for clients to a specific set of IP addresses
      or hostnames prior to authentication.
    type: bool
  urlRedirectWalledGardenRanges:
    description: IP address ranges, in CIDR notation, to restrict access for clients
      to a specific set of IP addresses or hostnames prior to authentication.
    elements: str
    type: list
  voiceVlanClients:
    description: CDP/LLDP capable voice clients will be able to use this VLAN. Automatically
      true when hostMode is 'Multi-Domain'.
    type: bool
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch createNetworkSwitchAccessPolicy
    description: Complete reference of the createNetworkSwitchAccessPolicy API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-switch-access-policy
  - name: Cisco Meraki documentation for switch deleteNetworkSwitchAccessPolicy
    description: Complete reference of the deleteNetworkSwitchAccessPolicy API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-switch-access-policy
  - name: Cisco Meraki documentation for switch updateNetworkSwitchAccessPolicy
    description: Complete reference of the updateNetworkSwitchAccessPolicy API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-access-policy
notes:
  - SDK Method used are
    switch.Switch.create_network_switch_access_policy,
    switch.Switch.delete_network_switch_access_policy,
    switch.Switch.update_network_switch_access_policy,
  - Paths used are
    post /networks/{networkId}/switch/accessPolicies,
    delete /networks/{networkId}/switch/accessPolicies/{accessPolicyNumber},
    put /networks/{networkId}/switch/accessPolicies/{accessPolicyNumber},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_switch_access_policies:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    accessPolicyType: Hybrid authentication
    dot1x:
      controlDirection: inbound
    guestGroupPolicyId: '1234'
    guestPortBouncing: false
    guestSgtId: 1284392014819
    guestVlanId: 100
    hostMode: Single-Host
    increaseAccessSpeed: false
    name: 'Access policy #1'
    networkId: string
    radius:
      authentication:
        mode: Open
      cache:
        enabled: false
        timeout: 24
      criticalAuth:
        dataGroupPolicyId: '1234'
        dataSgtId: 1234
        dataVlanId: 100
        suspendPortBounce: true
        voiceGroupPolicyId: '1234'
        voiceSgtId: 1234
        voiceVlanId: 100
      failedAuthGroupPolicyId: '1234'
      failedAuthSgtId: 1284392014819
      failedAuthVlanId: 100
      preAuthenticationGroupPolicyId: '1234'
      reAuthenticationInterval: 120
    radiusAccountingEnabled: true
    radiusAccountingServers:
      - host: 1.2.3.4
        organizationRadiusServerId: '42'
        port: 22
        secret: secret
    radiusCoaSupportEnabled: false
    radiusGroupAttribute: '11'
    radiusServers:
      - host: 1.2.3.4
        organizationRadiusServerId: '42'
        port: 22
        secret: secret
    radiusTestingEnabled: false
    urlRedirectWalledGardenEnabled: true
    urlRedirectWalledGardenRanges:
      - 192.168.1.0/24
    voiceVlanClients: true
- name: Update by id
  cisco.meraki.networks_switch_access_policies:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    accessPolicyNumber: string
    accessPolicyType: Hybrid authentication
    dot1x:
      controlDirection: inbound
    guestGroupPolicyId: '1234'
    guestPortBouncing: false
    guestSgtId: 1284392014819
    guestVlanId: 100
    hostMode: Single-Host
    increaseAccessSpeed: false
    name: 'Access policy #1'
    networkId: string
    radius:
      authentication:
        mode: Open
      cache:
        enabled: false
        timeout: 24
      criticalAuth:
        dataGroupPolicyId: '1234'
        dataSgtId: 1234
        dataVlanId: 100
        suspendPortBounce: true
        voiceGroupPolicyId: '1234'
        voiceSgtId: 1234
        voiceVlanId: 100
      failedAuthGroupPolicyId: '1234'
      failedAuthSgtId: 1284392014819
      failedAuthVlanId: 100
      preAuthenticationGroupPolicyId: '1234'
      reAuthenticationInterval: 120
    radiusAccountingEnabled: true
    radiusAccountingServers:
      - host: 1.2.3.4
        organizationRadiusServerId: '42'
        port: 22
        secret: secret
        serverId: '2'
    radiusCoaSupportEnabled: false
    radiusGroupAttribute: '11'
    radiusServers:
      - host: 1.2.3.4
        organizationRadiusServerId: '42'
        port: 22
        secret: secret
        serverId: '1'
    radiusTestingEnabled: false
    urlRedirectWalledGardenEnabled: true
    urlRedirectWalledGardenRanges:
      - 192.168.1.0/24
    voiceVlanClients: true
- name: Delete by id
  cisco.meraki.networks_switch_access_policies:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    accessPolicyNumber: string
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "accessPolicyNumber": "string",
      "name": "string",
      "radiusServers": [
        {
          "serverId": "string",
          "organizationRadiusServerId": "string",
          "host": "string",
          "port": 0
        }
      ],
      "radius": {
        "criticalAuth": {
          "dataVlanId": 0,
          "voiceVlanId": 0,
          "suspendPortBounce": true,
          "dataGroupPolicyId": "string",
          "voiceGroupPolicyId": "string",
          "dataSgtId": 0,
          "voiceSgtId": 0
        },
        "failedAuthVlanId": 0,
        "failedAuthGroupPolicyId": "string",
        "failedAuthSgtId": 0,
        "reAuthenticationInterval": 0,
        "cache": {
          "enabled": true,
          "timeout": 0
        },
        "authentication": {
          "mode": "string"
        },
        "preAuthenticationGroupPolicyId": "string"
      },
      "enforceRadiusMonitoring": true,
      "guestPortBouncing": true,
      "radiusTestingEnabled": true,
      "radiusCoaSupportEnabled": true,
      "radiusAccountingEnabled": true,
      "radiusAccountingServers": [
        {
          "serverId": "string",
          "organizationRadiusServerId": "string",
          "host": "string",
          "port": 0
        }
      ],
      "radiusGroupAttribute": "string",
      "hostMode": "string",
      "accessPolicyType": "string",
      "increaseAccessSpeed": true,
      "guestVlanId": {},
      "dot1x": {
        "controlDirection": "string"
      },
      "voiceVlanClients": true,
      "urlRedirectWalledGardenEnabled": true,
      "urlRedirectWalledGardenRanges": [
        "string"
      ],
      "counts": {
        "ports": {
          "withThisPolicy": 0
        }
      },
      "guestGroupPolicyId": "string",
      "guestSgtId": 0
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Shreyas Srish <ssrish@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: aci_node_mgmt_epg
short_description: In band or Out of band management EPGs (mgmt:OoB and mgmt:InB)
description:
- Cisco ACI Fabric Node EPGs
options:
  epg:
    description:
    - The name of the end point group
    type: str
    aliases: [ name ]
  type:
    description:
    - type of management interface
    type: str
    choices: [ in_band, out_of_band ]
    required: true
  bd:
    description:
    - The in-band bridge domain which is used when type is in_band
    type: str
  encap:
    description:
    - The in-band access encapsulation which is used when type is in_band
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(mgmt:OoB) and B(mgmt:InB).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Add in band mgmt epg
  cisco.aci.aci_node_mgmt_epg:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    epg: default
    type: in_band
    encap: vlan-1
    bd: bd1
    state: present
  delegate_to: localhost

- name: Add out of band mgmt epg
  cisco.aci.aci_node_mgmt_epg:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    epg: default
    type: out_of_band
    state: present
  delegate_to: localhost

- name: Query in band mgmt epg
  cisco.aci.aci_node_mgmt_epg:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    epg: default
    type: in_band
    encap: vlan-1
    bd: bd1
    state: query
  delegate_to: localhost

- name: Query all in band mgmt epg
  cisco.aci.aci_node_mgmt_epg:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    type: in_band
    state: query
  delegate_to: localhost

- name: Query all out of band mgmt epg
  cisco.aci.aci_node_mgmt_epg:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    type: out_of_band
    state: query
  delegate_to: localhost

- name: Remove in band mgmt epg
  cisco.aci.aci_node_mgmt_epg:
    host: "Host IP"
    username: admin
    password: SomeSecretePassword
    epg: default
    type: in_band
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: class_map (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        type=dict(type="str", choices=["in_band", "out_of_band"], required=True),
        epg=dict(type="str", aliases=["name"]),
        bd=dict(type="str"),
        encap=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True, required_if=[["state", "absent", ["epg"]], ["state", "present", ["epg"]]])

    type = module.params.get("type")
    epg = module.params.get("epg")
    bd = module.params.get("bd")
    encap = module.params.get("encap")
    state = module.params.get("state")

    child_configs = []
    child_class = []
    if type == "in_band":
        child_configs = [
            dict(
                mgmtRsMgmtBD=dict(
                    attributes=dict(
                        tnFvBDName=bd,
                    ),
                ),
            )
        ]

        child_class = ["mgmtRsMgmtBD"]

    class_map = dict(
        in_band=list(
            [
                dict(aci_class="mgmtInB", aci_rn="inb-{0}"),
            ]
        ),
        out_of_band=list(
            [
                dict(aci_class="mgmtOoB", aci_rn="oob-{0}"),
            ]
        ),
    )

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-mgmt",
            module_object="mgmt",
            target_filter={"name": "mgmt"},
        ),
        subclass_1=dict(
            aci_class="mgmtMgmtP",
            aci_rn="mgmtp-default",
            module_object="default",
            target_filter={"name": "default"},
        ),
        subclass_2=dict(
            aci_class=class_map.get(type)[0]["aci_class"],
            aci_rn=class_map.get(type)[0]["aci_rn"].format(epg),
            module_object=epg,
            target_filter={"name": epg},
        ),
        child_classes=child_class,
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class=class_map.get(type)[0]["aci_class"],
            class_config=dict(
                name=epg,
                encap=encap,
            ),
            child_configs=child_configs,
        )
        aci.get_diff(aci_class=class_map.get(type)[0]["aci_class"])

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Akini Ross (@akiniross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template_vrf_rp
short_description: Manage VRF Rendezvous Points (RP) in schema templates.
description:
- Manage VRF Rendezvous Points (RP) in schema templates on Cisco ACI Multi-Site.
- RPs can only be configured when the VRF has Layer 3 Multicast is enabled.
author:
- Akini Ross (@akinross)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  template:
    description:
    - The name of the template.
    type: str
    required: true
  vrf:
    description:
    - The name of the VRF.
    type: str
    required: true
  ip:
    description:
    - The IP address of the RP.
    type: str
    aliases: [ ip_address ]
  type:
    description:
    - The type of the RP.
    type: str
    choices: [ fabric, static ]
  multicast_route_map_policy:
    description:
    - The name of the Multicast Route Map Policy.
    - The Multicast Route Map Policy must reside in the same tenant as the tenant associated to the schema.
    - Use O(multicast_route_map_policy="") to remove the Multicast Route Map Policy.
    type: str
    aliases: [ route_map_policy ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new VRF RP
  cisco.mso.mso_schema_template_vrf_rp:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    vrf: VRF 1
    ip: 1.1.1.1
    type: static
    multicast_route_map_policy: RouteMapPolicy1
    state: present

- name: Query a specific VRF RP
  cisco.mso.mso_schema_template_vrf_rp:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    vrf: VRF 1
    ip: 1.1.1.1
    state: query
  register: query_result

- name: Query all VRF RPs
  cisco.mso.mso_schema_template_vrf_rp:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    vrf: VRF 1
    state: query
  register: query_result

- name: Remove an VRF RP
  cisco.mso.mso_schema_template_vrf_rp:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    vrf: VRF 1
    ip: 1.1.1.1
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.schema import MSOSchema
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        template=dict(type="str", required=True),
        vrf=dict(type="str", required=True),
        ip=dict(type="str", aliases=["ip_address"]),
        type=dict(type="str", choices=["fabric", "static"]),
        multicast_route_map_policy=dict(type="str", aliases=["route_map_policy"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["ip"]],
            ["state", "present", ["ip", "type"]],
        ],
    )

    schema = module.params.get("schema")
    template = module.params.get("template").replace(" ", "")
    vrf = module.params.get("vrf")
    ip = module.params.get("ip")
    rp_type = module.params.get("type")
    multicast_route_map_policy = module.params.get("multicast_route_map_policy")
    state = module.params.get("state")

    rps_path = "/templates/{0}/vrfs/{1}/rpConfigs".format(template, vrf)

    mso = MSOModule(module)

    mso_schema = MSOSchema(mso, schema, template)
    mso_schema.set_template(template)
    mso_schema.set_template_vrf(vrf)

    if state == "query":
        if ip:
            mso_schema.set_template_vrf_rp(ip)
            mso.existing = mso_schema.schema_objects.get("template_vrf_rp").details
        else:
            mso.existing = mso_schema.schema_objects.get("template_vrf").details.get("rpConfigs", [])
        mso.exit_json()

    mso_schema.set_template_vrf_rp(ip, False)

    rp = mso_schema.schema_objects.get("template_vrf_rp")
    ops = []

    mso.existing = mso.previous = rp.details if rp else mso.existing

    if state == "absent":
        if mso.existing:
            mso.sent = mso.existing = {}
            ops.append(dict(op="remove", path="{0}/{1}".format(rps_path, rp.index)))

    elif state == "present":

        payload = dict(ipAddress=ip, rpType=rp_type)

        if multicast_route_map_policy:
            payload["mcastRtMapPolicyRef"] = get_route_map_uuid(
                mso, mso_schema.schema_objects.get("template").details.get("tenantId"), multicast_route_map_policy
            )

        mso.sanitize(payload, collate=True)

        if mso.existing:
            rp_path = "{0}/{1}".format(rps_path, rp.index)
            if rp_type != mso.existing.get("rpType"):
                ops.append(dict(op="replace", path=rp_path + "/rpType", value=rp_type))
            if multicast_route_map_policy and payload.get("mcastRtMapPolicyRef") != mso.existing.get("mcastRtMapPolicyRef"):
                ops.append(
                    dict(
                        op="replace" if mso.existing.get("mcastRtMapPolicyRef") is not None else "add",
                        path=rp_path + "/mcastRtMapPolicyRef",
                        value=payload.get("mcastRtMapPolicyRef"),
                    )
                )
            if multicast_route_map_policy == "":
                ops.append(dict(op="remove", path=rp_path + "/mcastRtMapPolicyRef"))
        else:
            ops.append(dict(op="add", path=rps_path + "/-", value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(mso_schema.path, method="PATCH", data=ops)

    mso.exit_json()


def get_route_map_uuid(mso, tenant_id, multicast_route_map_policy):

    # Only tenant type templates contain the correct route map policies
    # Retrieves the list templates that contain the same tenant because only route map policies for the tenant assigned to the schema are options to be chosen
    templates = [
        MSOTemplate(mso, template_id=template.get("templateId")) for template in MSOTemplate(mso, "tenant").template if template.get("tenantId") == tenant_id
    ]

    # NDO restricts route map policies in the same tenant to have the same name thus we can loop through the route map policies to find the correct uuid
    for template in templates:
        for route_map_policy in template.template.get("tenantPolicyTemplate", {}).get("template", {}).get("mcastRouteMapPolicies", []):
            if route_map_policy.get("name") == multicast_route_map_policy:
                return route_map_policy.get("uuid")

    mso.fail_json(msg="Multicast Route Map Policy '{0}' not found.".format(multicast_route_map_policy))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

DOCUMENTATION = r"""
---
module: autoscaling_instance_info
version_added: 9.0.0
short_description: describe instances associated with AWS AutoScaling Groups (ASGs)
description:
  - Describe instances associated with AWS AutoScaling Groups (ASGs).
author:
  - "Mark Chappell (@tremble)"
options:
  group_name:
    description:
      - Name of the AutoScaling Group to manage.
      - O(group_name) and O(instance_ids) are mutually exclusive.
    type: str
  instance_ids:
    description:
      - The IDs of the EC2 instances.
      - O(group_name) and O(instance_ids) are mutually exclusive.
    type: list
    elements: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Describe all instances in a region
  amazon.aws.autoscaling_instance_info:
  register: instances

- name: Describe a specific instance
  amazon.aws.autoscaling_instance_info:
    instance_ids:
      - "i-123456789abcdef01"
  register: instances

- name: Describe the instances attached to a specific Auto Scaling Group
  amazon.aws.autoscaling_instance_info:
    group_name: example-asg
  register: instances
"""

RETURN = r"""
auto_scaling_instances:
  description: A description of the EC2 instances attached to an Auto Scaling group.
  returned: always
  type: list
  contains:
    availability_zone:
      description: The availability zone that the instance is in.
      returned: always
      type: str
      sample: "us-east-1a"
    health_status:
      description: The last reported health status of the instance.
      returned: always
      type: str
      sample: "Healthy"
    instance_id:
      description: The ID of the instance.
      returned: always
      type: str
      sample: "i-123456789abcdef01"
    instance_type:
      description: The instance type of the instance.
      returned: always
      type: str
      sample: "t3.micro"
    launch_configuration_name:
      description: The name of the launch configuration used when launching the instance.
      returned: When the instance was launched using an Auto Scaling launch configuration.
      type: str
      sample: "ansible-test-49630214-mchappel-thinkpadt14gen3-asg-instance-1"
    launch_template:
      description: A description of the launch template used when launching the instance.
      returned: When the instance was launched using an Auto Scaling launch template.
      type: dict
      contains:
        launch_template_id:
          description: The ID of the launch template used when launching the instance.
          returned: always
          type: str
          sample: "12345678-abcd-ef12-2345-6789abcdef01"
        launch_template_name:
          description: The name of the launch template used when launching the instance.
          returned: always
          type: str
          sample: "example-launch-configuration"
        version:
          description: The version of the launch template used when launching the instance.
          returned: always
          type: str
          sample: "$Default"
    lifecycle_state:
      description: The lifecycle state of the instance.
      returned: always
      type: str
      sample: "InService"
    protected_from_scale_in:
      description: Whether the instance is protected from termination when the Auto Scaling group is scaled in.
      returned: always
      type: bool
      sample: false
"""

from ansible_collections.amazon.aws.plugins.module_utils.autoscaling import get_autoscaling_instances
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


def main():
    argument_spec = dict(
        group_name=dict(type="str"),
        instance_ids=dict(type="list", elements="str"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[["instance_ids", "group_name"]],
    )

    client = module.client("autoscaling", retry_decorator=AWSRetry.jittered_backoff())

    instances = get_autoscaling_instances(
        client,
        instance_ids=module.params["instance_ids"],
        group_name=module.params["group_name"],
    )

    module.exit_json(changed=False, auto_scaling_instances=instances)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2016, Roman Belyakovsky <ihryamzik () gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: interfaces_file
short_description: Tweak settings in C(/etc/network/interfaces) files
extends_documentation_fragment:
  - ansible.builtin.files
  - community.general._attributes
description:
  - Manage (add, remove, change) individual interface options in an interfaces-style file without having to manage the file
    as a whole with, say, M(ansible.builtin.template) or M(ansible.builtin.assemble). Interface has to be presented in a file.
  - Read information about interfaces from interfaces-styled files.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  dest:
    type: path
    description:
      - Path to the interfaces file.
    default: /etc/network/interfaces
  iface:
    type: str
    description:
      - Name of the interface, required for value changes or option remove.
  address_family:
    type: str
    description:
      - Address family of the interface, useful if same interface name is used for both V(inet) and V(inet6).
  option:
    type: str
    description:
      - Name of the option, required for value changes or option remove.
  value:
    type: str
    description:
      - If O(option) is not presented for the O(iface) and O(state) is V(present), then O(option) is added. If O(option) already
        exists and is not V(pre-up), V(up), V(post-up) or V(down), its value is updated. V(pre-up), V(up), V(post-up) and
        V(down) options cannot be updated, only adding new options, removing existing ones or cleaning the whole option set
        are supported.
  backup:
    description:
      - Create a backup file including the timestamp information so you can get the original file back if you somehow clobbered
        it incorrectly.
    type: bool
    default: false
  state:
    type: str
    description:
      - If set to V(absent) the option or section is removed if present instead of created.
    default: "present"
    choices: ["present", "absent"]

notes:
  - If option is defined multiple times last one is updated but all others are deleted in case of an O(state=absent).
requirements: []
author: "Roman Belyakovsky (@hryamzik)"
"""

RETURN = r"""
dest:
  description: Destination file/path.
  returned: success
  type: str
  sample: "/etc/network/interfaces"
ifaces:
  description: Interfaces dictionary.
  returned: success
  type: dict
  contains:
    ifaces:
      description: Interface dictionary.
      returned: success
      type: dict
      contains:
        eth0:
          description: Name of the interface.
          returned: success
          type: dict
          contains:
            address_family:
              description: Interface address family.
              returned: success
              type: str
              sample: "inet"
            method:
              description: Interface method.
              returned: success
              type: str
              sample: "manual"
            mtu:
              description: Other options, all values returned as strings.
              returned: success
              type: str
              sample: "1500"
            pre-up:
              description: List of C(pre-up) scripts.
              returned: success
              type: list
              elements: str
              sample:
                - "route add -net 10.10.10.0/24 gw 10.10.10.1 dev eth1"
                - "route add -net 10.10.11.0/24 gw 10.10.11.1 dev eth2"
            up:
              description: List of C(up) scripts.
              returned: success
              type: list
              elements: str
              sample:
                - "route add -net 10.10.10.0/24 gw 10.10.10.1 dev eth1"
                - "route add -net 10.10.11.0/24 gw 10.10.11.1 dev eth2"
            post-up:
              description: List of C(post-up) scripts.
              returned: success
              type: list
              elements: str
              sample:
                - "route add -net 10.10.10.0/24 gw 10.10.10.1 dev eth1"
                - "route add -net 10.10.11.0/24 gw 10.10.11.1 dev eth2"
            down:
              description: List of C(down) scripts.
              returned: success
              type: list
              elements: str
              sample:
                - "route del -net 10.10.10.0/24 gw 10.10.10.1 dev eth1"
                - "route del -net 10.10.11.0/24 gw 10.10.11.1 dev eth2"
"""

EXAMPLES = r"""
- name: Set eth1 mtu configuration value to 8000
  community.general.interfaces_file:
    dest: /etc/network/interfaces.d/eth1.cfg
    iface: eth1
    option: mtu
    value: 8000
    backup: true
    state: present
  register: eth1_cfg
"""

import os
import re
import tempfile

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_bytes


def lineDict(line):
    return {"line": line, "line_type": "unknown"}


def optionDict(line, iface, option, value, address_family):
    return {
        "line": line,
        "iface": iface,
        "option": option,
        "value": value,
        "line_type": "option",
        "address_family": address_family,
    }


def getValueFromLine(s):
    option = s.split()[0]
    optionStart = s.find(option)
    optionLen = len(option)
    return s[optionLen + optionStart :].strip()


def read_interfaces_file(module, filename):
    with open(filename) as f:
        return read_interfaces_lines(module, f)


def read_interfaces_lines(module, line_strings):
    lines = []
    ifaces = {}
    currently_processing = None
    i = 0
    for line in line_strings:
        i += 1
        words = line.split()
        if len(words) < 1:
            lines.append(lineDict(line))
            continue
        if words[0][0] == "#":
            lines.append(lineDict(line))
            continue
        if words[0] == "mapping":
            # currmap = calloc(1, sizeof *currmap);
            lines.append(lineDict(line))
            currently_processing = "MAPPING"
        elif words[0] == "source":
            lines.append(lineDict(line))
            currently_processing = "NONE"
        elif words[0] == "source-dir":
            lines.append(lineDict(line))
            currently_processing = "NONE"
        elif words[0] == "source-directory":
            lines.append(lineDict(line))
            currently_processing = "NONE"
        elif words[0] == "iface":
            currif = {"pre-up": [], "up": [], "down": [], "post-up": []}
            iface_name = words[1]
            try:
                currif["address_family"] = words[2]
            except IndexError:
                currif["address_family"] = None
            address_family = currif["address_family"]
            try:
                currif["method"] = words[3]
            except IndexError:
                currif["method"] = None

            ifaces[iface_name] = currif
            lines.append(
                {
                    "line": line,
                    "iface": iface_name,
                    "line_type": "iface",
                    "params": currif,
                    "address_family": address_family,
                }
            )
            currently_processing = "IFACE"
        elif words[0] == "auto":
            lines.append(lineDict(line))
            currently_processing = "NONE"
        elif words[0].startswith("allow-"):
            lines.append(lineDict(line))
            currently_processing = "NONE"
        elif words[0] == "no-auto-down":
            lines.append(lineDict(line))
            currently_processing = "NONE"
        elif words[0] == "no-scripts":
            lines.append(lineDict(line))
            currently_processing = "NONE"
        else:
            if currently_processing == "IFACE":
                option_name = words[0]
                value = getValueFromLine(line)
                lines.append(optionDict(line, iface_name, option_name, value, address_family))
                if option_name in ["pre-up", "up", "down", "post-up"]:
                    currif[option_name].append(value)
                else:
                    currif[option_name] = value
            elif currently_processing == "MAPPING":
                lines.append(lineDict(line))
            elif currently_processing == "NONE":
                lines.append(lineDict(line))
            else:
                module.fail_json(msg=f"misplaced option {line} in line {i}")
                return None, None
    return lines, ifaces


def get_interface_options(iface_lines):
    return [i for i in iface_lines if i["line_type"] == "option"]


def get_target_options(iface_options, option):
    return [i for i in iface_options if i["option"] == option]


def update_existing_option_line(target_option, value):
    old_line = target_option["line"]
    old_value = target_option["value"]
    prefix_start = old_line.find(target_option["option"])
    optionLen = len(target_option["option"])
    old_value_position = re.search(r"\s+".join(map(re.escape, old_value.split())), old_line[prefix_start + optionLen :])
    start = old_value_position.start() + prefix_start + optionLen
    end = old_value_position.end() + prefix_start + optionLen
    line = old_line[:start] + value + old_line[end:]
    return line


def set_interface_option(module, lines, iface, option, raw_value, state, address_family=None):
    value = str(raw_value)
    changed = False

    iface_lines = [item for item in lines if "iface" in item and item["iface"] == iface]
    if address_family is not None:
        iface_lines = [
            item for item in iface_lines if "address_family" in item and item["address_family"] == address_family
        ]

    if len(iface_lines) < 1:
        # interface not found
        module.fail_json(msg=f"Error: interface {iface} not found")
        return changed, None

    iface_options = get_interface_options(iface_lines)
    target_options = get_target_options(iface_options, option)

    if state == "present":
        if len(target_options) < 1:
            changed = True
            # add new option
            last_line_dict = iface_lines[-1]
            changed, lines = addOptionAfterLine(
                option, value, iface, lines, last_line_dict, iface_options, address_family
            )
        else:
            if option in ["pre-up", "up", "down", "post-up"]:
                if len([i for i in target_options if i["value"] == value]) < 1:
                    changed, lines = addOptionAfterLine(
                        option, value, iface, lines, target_options[-1], iface_options, address_family
                    )
            else:
                # if more than one option found edit the last one
                if target_options[-1]["value"] != value:
                    changed = True
                    target_option = target_options[-1]
                    line = update_existing_option_line(target_option, value)
                    address_family = target_option["address_family"]
                    index = len(lines) - lines[::-1].index(target_option) - 1
                    lines[index] = optionDict(line, iface, option, value, address_family)
    elif state == "absent":
        if len(target_options) >= 1:
            if option in ["pre-up", "up", "down", "post-up"] and value is not None and value != "None":
                for target_option in [ito for ito in target_options if ito["value"] == value]:
                    changed = True
                    lines = [ln for ln in lines if ln != target_option]
            else:
                changed = True
                for target_option in target_options:
                    lines = [ln for ln in lines if ln != target_option]
    else:
        module.fail_json(msg=f"Error: unsupported state {state}, has to be either present or absent")

    return changed, lines


def addOptionAfterLine(option, value, iface, lines, last_line_dict, iface_options, address_family):
    # Changing method of interface is not an addition
    if option == "method":
        changed = False
        for ln in lines:
            if (
                ln.get("line_type", "") == "iface"
                and ln.get("iface", "") == iface
                and value != ln.get("params", {}).get("method", "")
            ):
                if address_family is not None and ln.get("address_family") != address_family:
                    continue
                changed = True
                ln["line"] = re.sub(f"{ln.get('params', {}).get('method', '')}$", value, ln.get("line"))
                ln["params"]["method"] = value
        return changed, lines

    last_line = last_line_dict["line"]
    prefix_start = last_line.find(last_line.split()[0])
    suffix_start = last_line.rfind(last_line.split()[-1]) + len(last_line.split()[-1])
    prefix = last_line[:prefix_start]

    if len(iface_options) < 1:
        # interface has no options, ident
        prefix += "    "

    line = f"{prefix}{option} {value}{last_line[suffix_start:]}"
    option_dict = optionDict(line, iface, option, value, address_family)
    index = len(lines) - lines[::-1].index(last_line_dict)
    lines.insert(index, option_dict)
    return True, lines


def write_changes(module, lines, dest):
    tmpfd, tmpfile = tempfile.mkstemp()
    with os.fdopen(tmpfd, "wb") as f:
        f.write(to_bytes("".join(lines), errors="surrogate_or_strict"))
    module.atomic_move(tmpfile, os.path.realpath(dest))


def main():
    module = AnsibleModule(
        argument_spec=dict(
            dest=dict(type="path", default="/etc/network/interfaces"),
            iface=dict(type="str"),
            address_family=dict(type="str"),
            option=dict(type="str"),
            value=dict(type="str"),
            backup=dict(type="bool", default=False),
            state=dict(type="str", default="present", choices=["absent", "present"]),
        ),
        add_file_common_args=True,
        supports_check_mode=True,
        required_by=dict(
            option=("iface",),
        ),
    )

    dest = module.params["dest"]
    iface = module.params["iface"]
    address_family = module.params["address_family"]
    option = module.params["option"]
    value = module.params["value"]
    backup = module.params["backup"]
    state = module.params["state"]

    if option is not None and state == "present" and value is None:
        module.fail_json(msg="Value must be set if option is defined and state is 'present'")

    lines, ifaces = read_interfaces_file(module, dest)

    changed = False

    if option is not None:
        changed, lines = set_interface_option(module, lines, iface, option, value, state, address_family)

    if changed:
        dummy, ifaces = read_interfaces_lines(module, [d["line"] for d in lines if "line" in d])

    if changed and not module.check_mode:
        if backup:
            module.backup_local(dest)
        write_changes(module, [d["line"] for d in lines if "line" in d], dest)

    module.exit_json(dest=dest, changed=changed, ifaces=ifaces)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2018 Hai Cao, <t-haicao@microsoft.com>, Yunge Zhu <yungez@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_cdnendpoint
version_added: "0.1.2"
short_description: Manage a Azure CDN endpoint
description:
    - Create, update, start, stop and delete a Azure CDN endpoint.

options:
    resource_group:
        description:
            - Name of a resource group where the Azure CDN endpoint exists or will be created.
        required: true
        type: str
    name:
        description:
            - Name of the Azure CDN endpoint.
        required: true
        type: str
    location:
        description:
            - Valid azure location. Defaults to location of the resource group.
        type: str
    started:
        description:
            - Use with I(state=present) to start the endpoint.
        type: bool
    purge:
        description:
            - Use with I(state=present) to purge the endpoint.
        type: bool
    purge_content_paths:
        description:
            - Use with I(state=present) and I(purge=true) to specify content paths to be purged.
        type: list
        elements: str
        default: ['/']
    profile_name:
        description:
            - Name of the CDN profile where the endpoint attached to.
        required: true
        type: str
    origins:
        description:
            - Set of source of the content being delivered via CDN.
            - Required when creating.
        elements: dict
        type: list
        suboptions:
            name:
                description:
                    - Origin name.
                required: true
                type: str
            host_name:
                description:
                    - The address of the origin.
                    - It can be a domain name, IPv4 address, or IPv6 address.
                required: true
                type: str
            http_port:
                description:
                    - The value of the HTTP port. Must be between C(1) and C(65535).
                type: int
            https_port:
                description:
                    - The value of the HTTPS port. Must be between C(1) and C(65535).
                type: int
    origin_host_header:
        description:
            - The host header value sent to the origin with each request.
        type: str
    origin_path:
        description:
            - A directory path on the origin that CDN can use to retrieve content from.
            - E.g. contoso.cloudapp.net/originpath.
        type: str
    content_types_to_compress:
        description:
            - List of content types on which compression applies.
            - This value should be a valid MIME type.
        type: list
        elements: str
    is_compression_enabled:
        description:
            - Indicates whether content compression is enabled on CDN.
        type: bool
        default: false
    is_http_allowed:
        description:
            - Indicates whether HTTP traffic is allowed on the endpoint.
        type: bool
        default: true
    is_https_allowed:
        description:
            - Indicates whether HTTPS traffic is allowed on the endpoint.
        type: bool
        default: true
    query_string_caching_behavior:
        description:
            - Defines how CDN caches requests that include query strings.
        type: str
        choices:
            - ignore_query_string
            - bypass_caching
            - use_query_string
            - not_set
        default: ignore_query_string
    state:
        description:
            - Assert the state of the Azure CDN endpoint. Use C(present) to create or update a Azure CDN endpoint and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Yunge Zhu (@yungezz)
'''

EXAMPLES = '''
- name: Create a Azure CDN endpoint
  azure_rm_cdnendpoint:
    resource_group: myResourceGroup
    profile_name: myProfile
    name: myEndpoint
    origins:
      - name: TestOrig
        host_name: "www.example.com"
    tags:
      testing: testing
      delete: on-exit
      foo: bar
- name: Delete a Azure CDN endpoint
  azure_rm_cdnendpoint:
    resource_group: myResourceGroup
    profile_name: myProfile
    name: myEndpoint
    state: absent
'''
RETURN = '''
state:
    description: Current state of the Azure CDN endpoint.
    returned: always
    type: str
id:
    description:
        - Id of the CDN endpoint.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myResourceGroup/providers/Microsoft.Cdn/profiles/myProfile/endpoints/
             myEndpoint"
host_name:
    description:
        - Host name of the CDN endpoint.
    returned: always
    type: str
    sample: "myendpoint.azureedge.net"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible.module_utils.common.dict_transformations import _snake_to_camel

try:
    from azure.mgmt.cdn.models import Endpoint, DeepCreatedOrigin, EndpointUpdateParameters, QueryStringCachingBehavior
except ImportError:
    # This is handled in azure_rm_common
    pass


def cdnendpoint_to_dict(cdnendpoint):
    return dict(
        id=cdnendpoint.id,
        name=cdnendpoint.name,
        type=cdnendpoint.type,
        location=cdnendpoint.location,
        tags=cdnendpoint.tags,
        origin_host_header=cdnendpoint.origin_host_header,
        origin_path=cdnendpoint.origin_path,
        content_types_to_compress=cdnendpoint.content_types_to_compress,
        is_compression_enabled=cdnendpoint.is_compression_enabled,
        is_http_allowed=cdnendpoint.is_http_allowed,
        is_https_allowed=cdnendpoint.is_https_allowed,
        query_string_caching_behavior=cdnendpoint.query_string_caching_behavior,
        optimization_type=cdnendpoint.optimization_type,
        probe_path=cdnendpoint.probe_path,
        geo_filters=[geo_filter_to_dict(geo_filter) for geo_filter in cdnendpoint.geo_filters] if cdnendpoint.geo_filters else None,
        host_name=cdnendpoint.host_name,
        origins=[deep_created_origin_to_dict(origin) for origin in cdnendpoint.origins] if cdnendpoint.origins else None,
        resource_state=cdnendpoint.resource_state,
        provisioning_state=cdnendpoint.provisioning_state
    )


def deep_created_origin_to_dict(origin):
    return dict(
        name=origin.name,
        host_name=origin.host_name,
        http_port=origin.http_port,
        https_port=origin.https_port,
    )


def geo_filter_to_dict(geo_filter):
    return dict(
        relative_path=geo_filter.relative_path,
        action=geo_filter.action,
        country_codes=geo_filter.country_codes,
    )


def default_content_types():
    return ["text/plain",
            "text/html",
            "text/css",
            "text/javascript",
            "application/x-javascript",
            "application/javascript",
            "application/json",
            "application/xml"]


origin_spec = dict(
    name=dict(
        type='str',
        required=True
    ),
    host_name=dict(
        type='str',
        required=True
    ),
    http_port=dict(
        type='int'
    ),
    https_port=dict(
        type='int'
    )
)


class AzureRMCdnendpoint(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
            started=dict(
                type='bool'
            ),
            purge=dict(
                type='bool'
            ),
            purge_content_paths=dict(
                type='list',
                elements='str',
                default=['/']
            ),
            profile_name=dict(
                type='str',
                required=True
            ),
            origins=dict(
                type='list',
                elements='dict',
                options=origin_spec
            ),
            origin_host_header=dict(
                type='str',
            ),
            origin_path=dict(
                type='str',
            ),
            content_types_to_compress=dict(
                type='list',
                elements='str',
            ),
            is_compression_enabled=dict(
                type='bool',
                default=False
            ),
            is_http_allowed=dict(
                type='bool',
                default=True
            ),
            is_https_allowed=dict(
                type='bool',
                default=True
            ),
            query_string_caching_behavior=dict(
                type='str',
                choices=[
                    'ignore_query_string',
                    'bypass_caching',
                    'use_query_string',
                    'not_set'
                ],
                default='ignore_query_string'
            ),
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.started = None
        self.purge = None
        self.purge_content_paths = None
        self.location = None
        self.profile_name = None
        self.origins = None
        self.tags = None
        self.origin_host_header = None
        self.origin_path = None
        self.content_types_to_compress = None
        self.is_compression_enabled = None
        self.is_http_allowed = None
        self.is_https_allowed = None
        self.query_string_caching_behavior = None

        self.results = dict(changed=False)

        super(AzureRMCdnendpoint, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 supports_tags=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        to_be_updated = False

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            self.location = resource_group.location

        if self.query_string_caching_behavior:
            self.query_string_caching_behavior = _snake_to_camel(self.query_string_caching_behavior, capitalize_first=True)

        response = self.get_cdnendpoint()

        if self.state == 'present':

            if not response:

                if self.started is None:
                    # If endpoint doesn't exist and no start/stop operation specified, create endpoint.
                    if self.origins is None:
                        self.fail("Origins is not provided when trying to create endpoint")
                    self.log("Need to create the Azure CDN endpoint")

                    if not self.check_mode:
                        result = self.create_cdnendpoint()
                        self.results['id'] = result['id']
                        self.results['host_name'] = result['host_name']
                        self.log("Creation done")

                    self.results['changed'] = True
                    return self.results

                else:
                    # Fail the module when user try to start/stop a non-existed endpoint
                    self.log("Can't stop/stop a non-existed endpoint")
                    self.fail("This endpoint is not found, stop/start is forbidden")

            else:
                self.log('Results : {0}'.format(response))
                self.results['id'] = response['id']
                self.results['host_name'] = response['host_name']

                update_tags, response['tags'] = self.update_tags(response['tags'])

                if update_tags:
                    to_be_updated = True

                if response['provisioning_state'] == "Succeeded":
                    if self.started is False and response['resource_state'] == 'Running':
                        self.log("Need to stop the Azure CDN endpoint")

                        if not self.check_mode:
                            result = self.stop_cdnendpoint()
                            self.log("Endpoint stopped")

                        self.results['changed'] = True

                    elif self.started and response['resource_state'] == 'Stopped':
                        self.log("Need to start the Azure CDN endpoint")

                        if not self.check_mode:
                            result = self.start_cdnendpoint()
                            self.log("Endpoint started")

                        self.results['changed'] = True

                    elif self.started is not None:
                        self.module.warn("Start/Stop not performed due to current resource state {0}".format(response['resource_state']))
                        self.results['changed'] = False

                    if self.purge:
                        self.log("Need to purge endpoint")

                        if not self.check_mode:
                            result = self.purge_cdnendpoint()
                            self.log("Endpoint purged")

                        self.results['changed'] = True

                    to_be_updated = to_be_updated or self.check_update(response)

                    if to_be_updated:
                        self.log("Need to update the Azure CDN endpoint")
                        self.results['changed'] = True

                        if not self.check_mode:
                            result = self.update_cdnendpoint()
                            self.results['host_name'] = result['host_name']
                            self.log("Update done")

                elif self.started is not None:
                    self.module.warn("Start/Stop not performed due to current provisioning state {0}".format(response['provisioning_state']))
                    self.results['changed'] = False

        elif self.state == 'absent' and response:
            self.log("Need to delete the Azure CDN endpoint")
            self.results['changed'] = True

            if not self.check_mode:
                self.delete_cdnendpoint()
                self.log("Azure CDN endpoint deleted")

        return self.results

    def create_cdnendpoint(self):
        '''
        Creates a Azure CDN endpoint.

        :return: deserialized Azure CDN endpoint instance state dictionary
        '''
        self.log("Creating the Azure CDN endpoint instance {0}".format(self.name))

        origins = []
        for item in self.origins:
            origins.append(
                DeepCreatedOrigin(name=item['name'],
                                  host_name=item['host_name'],
                                  http_port=item['http_port'] if 'http_port' in item else None,
                                  https_port=item['https_port'] if 'https_port' in item else None)
            )

        parameters = Endpoint(
            origins=origins,
            location=self.location,
            tags=self.tags,
            origin_host_header=self.origin_host_header,
            origin_path=self.origin_path,
            content_types_to_compress=default_content_types() if self.is_compression_enabled and not self.content_types_to_compress
            else self.content_types_to_compress,
            is_compression_enabled=self.is_compression_enabled if self.is_compression_enabled is not None else False,
            is_http_allowed=self.is_http_allowed if self.is_http_allowed is not None else True,
            is_https_allowed=self.is_https_allowed if self.is_https_allowed is not None else True,
            query_string_caching_behavior=self.query_string_caching_behavior if self.query_string_caching_behavior
            else QueryStringCachingBehavior.ignore_query_string
        )

        try:
            poller = self.cdn_client.endpoints.begin_create(self.resource_group, self.profile_name, self.name, parameters)
            response = self.get_poller_result(poller)
            return cdnendpoint_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to create Azure CDN endpoint instance.')
            self.fail("Error creating Azure CDN endpoint instance: {0}".format(exc.message))

    def update_cdnendpoint(self):
        '''
        Updates a Azure CDN endpoint.

        :return: deserialized Azure CDN endpoint instance state dictionary
        '''
        self.log("Updating the Azure CDN endpoint instance {0}".format(self.name))

        endpoint_update_properties = EndpointUpdateParameters(
            tags=self.tags,
            origin_host_header=self.origin_host_header,
            origin_path=self.origin_path,
            content_types_to_compress=default_content_types() if self.is_compression_enabled and not self.content_types_to_compress
            else self.content_types_to_compress,
            is_compression_enabled=self.is_compression_enabled,
            is_http_allowed=self.is_http_allowed,
            is_https_allowed=self.is_https_allowed,
            query_string_caching_behavior=self.query_string_caching_behavior,
        )

        try:
            poller = self.cdn_client.endpoints.begin_update(self.resource_group, self.profile_name, self.name, endpoint_update_properties)
            response = self.get_poller_result(poller)
            return cdnendpoint_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to update Azure CDN endpoint instance.')
            self.fail("Error updating Azure CDN endpoint instance: {0}".format(exc.message))

    def delete_cdnendpoint(self):
        '''
        Deletes the specified Azure CDN endpoint in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Azure CDN endpoint {0}".format(self.name))
        try:
            poller = self.cdn_client.endpoints.begin_delete(self.resource_group, self.profile_name, self.name)
            self.get_poller_result(poller)
            return True
        except Exception as e:
            self.log('Error attempting to delete the Azure CDN endpoint.')
            self.fail("Error deleting the Azure CDN endpoint: {0}".format(e.message))
            return False

    def get_cdnendpoint(self):
        '''
        Gets the properties of the specified Azure CDN endpoint.

        :return: deserialized Azure CDN endpoint state dictionary
        '''
        self.log(
            "Checking if the Azure CDN endpoint {0} is present".format(self.name))
        try:
            response = self.cdn_client.endpoints.get(self.resource_group, self.profile_name, self.name)
            self.log("Response : {0}".format(response))
            self.log("Azure CDN endpoint : {0} found".format(response.name))
            return cdnendpoint_to_dict(response)
        except Exception:
            self.log('Did not find the Azure CDN endpoint.')
            return False

    def start_cdnendpoint(self):
        '''
        Starts an existing Azure CDN endpoint that is on a stopped state.

        :return: deserialized Azure CDN endpoint state dictionary
        '''
        self.log(
            "Starting the Azure CDN endpoint {0}".format(self.name))
        try:
            poller = self.cdn_client.endpoints.begin_start(self.resource_group, self.profile_name, self.name)
            response = self.get_poller_result(poller)
            self.log("Response : {0}".format(response))
            self.log("Azure CDN endpoint : {0} started".format(response.name))
            return self.get_cdnendpoint()
        except Exception:
            self.log('Fail to start the Azure CDN endpoint.')
            return False

    def purge_cdnendpoint(self):
        '''
        Purges an existing Azure CDN endpoint.

        :return: deserialized Azure CDN endpoint state dictionary
        '''
        self.log(
            "Purging the Azure CDN endpoint {0}".format(self.name))
        try:
            poller = self.cdn_client.endpoints.begin_purge_content(self.resource_group,
                                                                   self.profile_name,
                                                                   self.name,
                                                                   content_file_paths=dict(content_paths=self.purge_content_paths))
            response = self.get_poller_result(poller)
            self.log("Response : {0}".format(response))
            return self.get_cdnendpoint()
        except Exception as e:
            self.fail('Fail to purge the Azure CDN endpoint.')
            return False

    def stop_cdnendpoint(self):
        '''
        Stops an existing Azure CDN endpoint that is on a running state.

        :return: deserialized Azure CDN endpoint state dictionary
        '''
        self.log(
            "Stopping the Azure CDN endpoint {0}".format(self.name))
        try:
            poller = self.cdn_client.endpoints.begin_stop(self.resource_group, self.profile_name, self.name)
            response = self.get_poller_result(poller)
            self.log("Response : {0}".format(response))
            self.log("Azure CDN endpoint : {0} stopped".format(response.name))
            return self.get_cdnendpoint()
        except Exception:
            self.log('Fail to stop the Azure CDN endpoint.')
            return False

    def check_update(self, response):

        if self.origin_host_header and response['origin_host_header'] != self.origin_host_header:
            self.log("Origin host header Diff - Origin {0} / Update {1}".format(response['origin_host_header'], self.origin_host_header))
            return True

        if self.origin_path and response['origin_path'] != self.origin_path:
            self.log("Origin path Diff - Origin {0} / Update {1}".format(response['origin_path'], self.origin_path))
            return True

        if self.content_types_to_compress and response['content_types_to_compress'] != self.content_types_to_compress:
            self.log("Content types to compress Diff - Origin {0} / Update {1}".format(response['content_types_to_compress'], self.content_types_to_compress))
            return True

        if self.is_compression_enabled is not None and response['is_compression_enabled'] != self.is_compression_enabled:
            self.log("is_compression_enabled Diff - Origin {0} / Update {1}".format(response['is_compression_enabled'], self.is_compression_enabled))
            return True

        if self.is_http_allowed is not None and response['is_http_allowed'] != self.is_http_allowed:
            self.log("is_http_allowed Diff - Origin {0} / Update {1}".format(response['is_http_allowed'], self.is_http_allowed))
            return True

        if self.is_https_allowed is not None and response['is_https_allowed'] != self.is_https_allowed:
            self.log("is_https_allowed Diff - Origin {0} / Update {1}".format(response['is_https_allowed'], self.is_https_allowed))
            return True

        if self.query_string_caching_behavior and \
           _snake_to_camel(response['query_string_caching_behavior']).lower() != _snake_to_camel(self.query_string_caching_behavior).lower():
            self.log("query_string_caching_behavior Diff - Origin {0} / Update {1}".format(response['query_string_caching_behavior'],
                                                                                           self.query_string_caching_behavior))
            return True

        return False


def main():
    """Main execution"""
    AzureRMCdnendpoint()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_qos_rules_order
short_description: Resource module for networks _switch _qos _rules _order
description:
  - Manage operations create, update and delete of the resource networks _switch _qos
    _rules _order.
  - Add a quality of service rule.
  - Delete a quality of service rule.
  - Update a quality of service rule.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  dscp:
    description: DSCP tag for the incoming packet. Set this to -1 to trust incoming
      DSCP. Default value is 0.
    type: int
  dstPort:
    description: The destination port of the incoming packet. Applicable only if protocol
      is TCP or UDP.
    type: int
  dstPortRange:
    description: The destination port range of the incoming packet. Applicable only
      if protocol is set to TCP or UDP.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  protocol:
    description: The protocol of the incoming packet. Default value is "ANY".
    type: str
  qosRuleId:
    description: QosRuleId path parameter. Qos rule ID.
    type: str
  srcPort:
    description: The source port of the incoming packet. Applicable only if protocol
      is TCP or UDP.
    type: int
  srcPortRange:
    description: The source port range of the incoming packet. Applicable only if
      protocol is set to TCP or UDP.
    type: str
  vlan:
    description: The VLAN of the incoming packet. A null value will match any VLAN.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch createNetworkSwitchQosRule
    description: Complete reference of the createNetworkSwitchQosRule API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-switch-qos-rule
  - name: Cisco Meraki documentation for switch deleteNetworkSwitchQosRule
    description: Complete reference of the deleteNetworkSwitchQosRule API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-switch-qos-rule
  - name: Cisco Meraki documentation for switch updateNetworkSwitchQosRule
    description: Complete reference of the updateNetworkSwitchQosRule API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-qos-rule
notes:
  - SDK Method used are
    switch.Switch.create_network_switch_qos_rule,
    switch.Switch.delete_network_switch_qos_rule,
    switch.Switch.update_network_switch_qos_rule,
  - Paths used are
    post /networks/{networkId}/switch/qosRules,
    delete /networks/{networkId}/switch/qosRules/{qosRuleId},
    put /networks/{networkId}/switch/qosRules/{qosRuleId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_switch_qos_rules_order:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    dscp: 0
    dstPort: 3000
    dstPortRange: 3000-3100
    networkId: string
    protocol: TCP
    srcPort: 2000
    srcPortRange: 70-80
    vlan: 100
- name: Delete by id
  cisco.meraki.networks_switch_qos_rules_order:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    networkId: string
    qosRuleId: string
- name: Update by id
  cisco.meraki.networks_switch_qos_rules_order:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    dscp: 0
    dstPort: 3000
    dstPortRange: 3000-3100
    networkId: string
    protocol: TCP
    qosRuleId: string
    srcPort: 2000
    srcPortRange: 70-80
    vlan: 100
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "vlan": {},
      "protocol": "string",
      "srcPort": 0,
      "srcPortRange": "string",
      "dstPort": 0,
      "dstPortRange": "string",
      "dscp": 0
    }
"""



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_tags_info

version_added: "3.5.0"

short_description: List tags facts

description:
    - List all tag details under subscription_id.
    - Get the tag info at the specified scope.

options:
    scope:
        description:
            - The resource scope.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get the tag info at thespecified resource
  azure_rm_tags_info:
    scope: scope_str

- name: List all tag details under subscription
  azure_rm_tags_info:
'''
RETURN = '''
tag_details:
    description:
        - List tag details.
    returned: when I(scope!=None)
    type: complex
    contains:
        id:
            description:
                - The tag name ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/tagNames/k1"
        tag_name:
            description:
                - The tag name.
            returned: always
            type: str
            sample: K1
        count:
            description:
                - The total number of resources that use the resource tag.
                -  When a tag is initially created and has no associated resources, the value is 0.
            returned: always
            type: dict
            sample: { 'type': 'Total', 'value': 1}
        tag_values:
            description:
                - The list of tag values.
            returned: always
            type: complex
            contains:
                id:
                    description:
                        - The tag value ID.
                    returned: always
                    type: str
                    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/tagNames/key2/tagValues/v1"
                tag_value:
                    description:
                        - The tag value.
                    returned: always
                    type: str
                    sample: V1
                count:
                    description:
                        - The tag value count
                    returned: always
                    type: dict
                    sample: {'type': 'Total', 'value': 1}
        values:
            description:
                - The list of tag values.
                - This return value has been deprecated and will be removed in a release after 2027-05-01.
                  Use C(tag_values) instead.
            returned: always
            type: complex
            contains:
                id:
                    description:
                        - The tag value ID.
                    returned: always
                    type: str
                    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/tagNames/key2/tagValues/v1"
                tag_value:
                    description:
                        - The tag value.
                    returned: always
                    type: str
                    sample: V1
                count:
                    description:
                        - The tag value count
                    returned: always
                    type: dict
                    sample: {'type': 'totoal', 'value': 1}
tag_info:
    description:
        - The tag info.
    returned: when I(scope=None)
    type: complex
    contains:
        id:
            description:
                - The ID of the tags wrapper resource.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Resources/tags/default"
        name:
            description:
                - The name of the tags wrapper resource.
            returned: always
            type: str
            sample: default
        type:
            description:
                - The type of the tags wrapper resource.
            returned: always
            type: str
            sample: Microsoft.Resources/tags
        properties:
            description:
                - The set of tags.
            returned: always
            type: dict
            sample: { 'tags': {'key1': 'value1', 'key2': 'value2'}}
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


AZURE_OBJECT_CLASS = 'Tags'


class AzureRMTagsInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            scope=dict(type='str'),
        )

        self.results = dict(
            changed=False,
            tags_detail=[],
            tag_info=None
        )

        self.scope = None

        super(AzureRMTagsInfo, self).__init__(self.module_arg_spec,
                                              supports_check_mode=True,
                                              supports_tags=False,
                                              facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.scope:
            self.results['tag_info'] = self.get_at_scope(self.scope)
        else:
            self.results['tag_details'] = self.list_all()
            self.module.deprecate(
                "The 'values' return key in 'tag_details' is deprecated. Use 'tag_values' instead.",
                date="2027-05-01",
                collection_name="azure.azcollection",
            )

        return self.results

    def get_at_scope(self, scope):
        self.log('Get properties for {0}'.format(self.scope))
        results = []
        try:
            response = self.rm_client.tags.get_at_scope(scope)
            if response:
                results.append(self.format_tags(response))
        except StopIteration:
            pass
        except Exception as exc:
            self.fail('Error when get the tags info under specified scope got Excetion as {0}'.format(exc.message or str(exc)))
        return results

    def list_all(self):
        self.log('List resources under resource group')
        results = []
        try:
            response = self.rm_client.tags.list()
            while True:
                tag_dict = response.next().as_dict()
                if 'values' in tag_dict:
                    tag_dict['tag_values'] = tag_dict['values']
                results.append(tag_dict)
        except StopIteration:
            pass
        except Exception as exc:
            self.fail('Error when listing all tags under subscription got Excetion as {0}'.format(exc.message or str(exc)))
        return results

    def format_tags(self, tags):
        results = dict(
            id=tags.id,
            name=tags.name,
            type=tags.type,
            properties=dict()
        )
        if tags.properties:
            results['properties'] = tags.properties.as_dict()

        return results


def main():
    AzureRMTagsInfo()


if __name__ == '__main__':
    main()



# Copyright (c) 2017-2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import traceback
import typing as t

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.dns.plugins.module_utils._argspec import (
    ArgumentSpec,
    ModuleOptionProvider,
)
from ansible_collections.community.dns.plugins.module_utils._conversion.base import (
    DNSConversionError,
)
from ansible_collections.community.dns.plugins.module_utils._conversion.converter import (
    RecordConverter,
)
from ansible_collections.community.dns.plugins.module_utils._options import (
    create_record_transformation_argspec,
)
from ansible_collections.community.dns.plugins.module_utils._record import (
    DNSRecord,
    format_record_for_output,
)
from ansible_collections.community.dns.plugins.module_utils._record_set import (
    DNSRecordSet,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    NOT_PROVIDED,
    DNSAPIAuthenticationError,
    DNSAPIError,
    ZoneRecordAPI,
    filter_records,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_set_api import (
    filter_record_sets,
)

from ._utils import get_prefix, normalize_dns_name

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover

    from .._provider import ProviderInformation  # pragma: no cover
    from .._record import IDNSRecord, RecordIDT  # pragma: no cover
    from .._record_set import RecordSetIDT  # pragma: no cover
    from .._zone import ZoneIDT  # pragma: no cover
    from .._zone_record_set_api import ZoneRecordSetAPI  # pragma: no cover


def create_module_argument_spec(
    provider_information: ProviderInformation,
) -> ArgumentSpec:
    return ArgumentSpec(
        argument_spec={
            "state": {
                "type": "str",
                "choices": ["present", "absent"],
                "required": True,
            },
            "zone_name": {"type": "str", "aliases": ["zone"]},
            "zone_id": {"type": provider_information.get_zone_id_type()},
            "record": {"type": "str"},
            "prefix": {"type": "str"},
            "ttl": {
                "type": "int",
                "default": provider_information.get_record_default_ttl(),
            },
            "type": {
                "choices": provider_information.get_supported_record_types(),
                "required": True,
            },
            "value": {"type": "str", "required": True},
        },
        required_one_of=[
            ("zone_name", "zone_id"),
            ("record", "prefix"),
        ],
        mutually_exclusive=[
            ("zone_name", "zone_id"),
            ("record", "prefix"),
        ],
    ).merge(create_record_transformation_argspec())


def _run_module_record_api(
    module: AnsibleModule,
    provider_information: ProviderInformation,
    record_converter: RecordConverter,
    record_in: str,
    prefix_in: str | None,
    type_in: str,
    api: ZoneRecordAPI[ZoneIDT, RecordIDT],
) -> t.NoReturn:
    # Get zone information
    if module.params.get("zone_name") is not None:
        zone_in = normalize_dns_name(module.params.get("zone_name"))
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )
        zone = api.get_zone_with_records_by_name(
            zone_in, prefix=prefix, record_type=type_in
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_id = zone.zone.id
        records = zone.records
    elif record_in is not None:
        zone = api.get_zone_with_records_by_id(
            module.params.get("zone_id"),
            record_type=type_in,
            prefix=(
                provider_information.normalize_prefix(prefix_in)
                if prefix_in is not None
                else NOT_PROVIDED
            ),
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_in = normalize_dns_name(zone.zone.name)
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )
        zone_id = zone.zone.id
        records = zone.records
    else:
        zone_id = module.params.get("zone_id")
        prefix = provider_information.normalize_prefix(prefix_in)
        records = api.get_zone_records(
            zone_id,
            record_type=type_in,
            prefix=prefix,
        )
        if records is None:
            module.fail_json(msg="Zone not found")
        zone_in = None
        record_in = None

    # Find matching records
    records = filter_records(records, prefix=prefix, record_type=type_in)
    record_converter.process_multiple_from_api(records)

    # Parse records
    value_in = module.params.get("value")
    value_in = record_converter.process_value_from_user(type_in, value_in)

    # Compare records
    existing_record = None
    exact_match = False
    ttl_in = module.params.get("ttl")
    for record in records:
        if record.target == value_in:
            existing_record = record
            exact_match = record.ttl == ttl_in
            break

    before = existing_record.clone() if existing_record else None
    after: IDNSRecord[RecordIDT | None] | None = before
    changed = False

    if module.params.get("state") == "present":
        if existing_record is None:
            # Create record
            new_record: DNSRecord[RecordIDT | None] = DNSRecord(
                record_id=None, record_type=type_in, target=value_in
            )
            new_record.prefix = prefix
            new_record.ttl = ttl_in
            after = new_record
            new_api_record = record_converter.clone_to_api(new_record)
            if not module.check_mode:
                created_api_record = api.add_record(zone_id, new_api_record)
                created_record = record_converter.clone_from_api(created_api_record)
                after = created_record
            changed = True
        elif not exact_match:
            # Update record
            record = existing_record
            record.ttl = ttl_in
            api_record = record_converter.clone_to_api(record)
            if not module.check_mode:
                api_record = api.update_record(zone_id, api_record)
                record = record_converter.clone_from_api(api_record)
            after = record
            changed = True
    else:
        if existing_record is not None:
            # Delete record
            api_record = record_converter.clone_to_api(existing_record)
            if not module.check_mode:
                api.delete_record(zone_id, api_record)
            after = None
            changed = True

    # Compose result
    result = {
        "changed": changed,
        "zone_id": zone_id,
    }
    if module._diff:
        result["diff"] = {
            "before": (
                format_record_for_output(
                    before, record_in, prefix, record_converter=record_converter
                )
                if before
                else {}
            ),
            "after": (
                format_record_for_output(
                    after, record_in, prefix, record_converter=record_converter
                )
                if after
                else {}
            ),
        }

    module.exit_json(**result)


def _run_module_record_set_api(
    module: AnsibleModule,
    provider_information: ProviderInformation,
    record_converter: RecordConverter,
    record_in: str,
    prefix_in: str | None,
    type_in: str,
    api: ZoneRecordSetAPI[ZoneIDT, RecordSetIDT, RecordIDT],
) -> t.NoReturn:
    # Get zone information
    if module.params.get("zone_name") is not None:
        zone_in = normalize_dns_name(module.params.get("zone_name"))
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )
        zone = api.get_zone_with_record_sets_by_name(
            zone_in, prefix=prefix, record_type=type_in
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_id = zone.zone.id
        record_sets = zone.record_sets
    elif record_in is not None:
        zone = api.get_zone_with_record_sets_by_id(
            module.params.get("zone_id"),
            record_type=type_in,
            prefix=(
                provider_information.normalize_prefix(prefix_in)
                if prefix_in is not None
                else NOT_PROVIDED
            ),
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_in = normalize_dns_name(zone.zone.name)
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )
        zone_id = zone.zone.id
        record_sets = zone.record_sets
    else:
        zone_id = module.params.get("zone_id")
        prefix = provider_information.normalize_prefix(prefix_in)
        record_sets = api.get_zone_record_sets(
            zone_id,
            record_type=type_in,
            prefix=prefix,
        )
        if record_sets is None:
            module.fail_json(msg="Zone not found")
        zone_in = None
        record_in = None

    # Find matching records
    record_sets = filter_record_sets(record_sets, prefix=prefix, record_type=type_in)
    for record_set_ in record_sets:
        record_converter.process_set_from_api(record_set_)
    record_set = record_sets[0] if record_sets else None
    records = list(record_set.records) if record_set else []

    # Parse records
    value_in = module.params.get("value")
    value_in = record_converter.process_value_from_user(type_in, value_in)

    # Compare records
    existing_record = None
    exact_match = False
    ttl_in = module.params.get("ttl")
    for record in records:
        if record.target == value_in:
            existing_record = record
            exact_match = record.ttl == ttl_in
            break

    before = existing_record.clone() if existing_record else None
    after = before
    changed = False

    if module.params.get("state") == "present":
        if existing_record is None:
            # Create record
            record = DNSRecord(record_id=None, record_type=type_in, target=value_in)
            record.prefix = prefix
            record.ttl = ttl_in
            after = record
            if record_set is None:
                # Create record set
                new_record_set: DNSRecordSet[RecordSetIDT | None, RecordIDT | None] = (
                    DNSRecordSet(record_set_id=None, record_type=type_in)
                )
                new_record_set.prefix = prefix
                new_record_set.ttl = ttl_in
                new_record_set.records.append(record)
                if not module.check_mode:
                    new_api_record_set = record_converter.clone_set_to_api(
                        new_record_set
                    )
                    record_set = api.add_record_set(zone_id, new_api_record_set)
            else:
                record_set.records.append(record)
                updated_ttl = record_set.ttl != ttl_in
                record_set.ttl = ttl_in
                if not module.check_mode:
                    api_record_set = record_converter.clone_set_to_api(record_set)
                    record_set = api.update_record_set(
                        zone_id,
                        api_record_set,
                        updated_records=True,
                        updated_ttl=updated_ttl,
                    )
            changed = True
            if not module.check_mode:
                assert record_set is not None
                for record in record_set.records:
                    if record.target == value_in:
                        after = record_converter.clone_from_api(record)
                        break
        elif not exact_match:
            assert record_set is not None
            # TTL needs updating
            record_set.ttl = ttl_in
            existing_record.ttl = ttl_in
            for record in record_set.records:
                record.ttl = ttl_in
            after = existing_record
            if not module.check_mode:
                api_record_set = record_converter.clone_set_to_api(record_set)
                record_set = api.update_record_set(
                    zone_id, api_record_set, updated_records=False, updated_ttl=True
                )
                for record in record_set.records:
                    if record.target == value_in:
                        after = record_converter.clone_from_api(record)
                        break
            changed = True
    else:
        if existing_record is not None:
            assert record_set is not None
            record_set.records.remove(existing_record)
            if not module.check_mode:
                api_record_set = record_converter.clone_set_to_api(record_set)
                if not record_set.records:
                    api.delete_record_set(zone_id, api_record_set)
                else:
                    record_set = api.update_record_set(
                        zone_id, api_record_set, updated_records=True, updated_ttl=False
                    )
            after = None
            changed = True

    # Compose result
    result = {
        "changed": changed,
        "zone_id": zone_id,
    }
    if module._diff:
        result["diff"] = {
            "before": (
                format_record_for_output(
                    before, record_in, prefix, record_converter=record_converter
                )
                if before
                else {}
            ),
            "after": (
                format_record_for_output(
                    after, record_in, prefix, record_converter=record_converter
                )
                if after
                else {}
            ),
        }

    module.exit_json(**result)


def run_module(
    module: AnsibleModule,
    create_api: t.Callable[[], ZoneRecordAPI | ZoneRecordSetAPI],
    provider_information: ProviderInformation,
) -> t.NoReturn:
    option_provider = ModuleOptionProvider(module)
    record_converter = RecordConverter(provider_information, option_provider)
    record_converter.emit_deprecations(module.deprecate)

    record_in = normalize_dns_name(module.params.get("record"))
    prefix_in = module.params.get("prefix")
    type_in = module.params.get("type")
    if type_in and type_in not in provider_information.get_supported_record_types():
        module.fail_json(msg=f"Invalid record type {type_in}")
    try:
        # Create API
        api = create_api()

        if isinstance(api, ZoneRecordAPI):
            _run_module_record_api(
                module,
                provider_information,
                record_converter,
                record_in,
                prefix_in,
                type_in,
                api,
            )
        else:
            _run_module_record_set_api(
                module,
                provider_information,
                record_converter,
                record_in,
                prefix_in,
                type_in,
                api,
            )

    except DNSConversionError as e:
        module.fail_json(
            msg=f"Error while converting DNS values: {e.error_message}",
            error=e.error_message,
            exception=traceback.format_exc(),
        )
    except DNSAPIAuthenticationError as e:
        module.fail_json(
            msg=f"Cannot authenticate: {e}",
            error=to_text(e),
            exception=traceback.format_exc(),
        )
    except DNSAPIError as e:
        module.fail_json(
            msg=f"Error: {e}",
            error=to_text(e),
            exception=traceback.format_exc(),
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---

version_added: 1.0.0
module: cloudfront_origin_access_identity

short_description: Create, update and delete origin access identities for a
                   CloudFront distribution

description:
  - Allows for easy creation, updating and deletion of origin access
    identities.

author:
  - Willem van Ketwich (@wilvk)

options:
    state:
      description: If the named resource should exist.
      choices:
        - present
        - absent
      default: present
      type: str
    origin_access_identity_id:
      description:
        - The origin_access_identity_id of the CloudFront distribution.
      required: false
      type: str
    comment:
      description:
        - A comment to describe the CloudFront origin access identity.
      required: false
      type: str
    caller_reference:
      description:
        - A unique identifier to reference the origin access identity by.
      required: false
      type: str

notes:
  - Does not support check mode.

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""

- name: create an origin access identity
  community.aws.cloudfront_origin_access_identity:
    state: present
    caller_reference: this is an example reference
    comment: this is an example comment

- name: update an existing origin access identity using caller_reference as an identifier
  community.aws.cloudfront_origin_access_identity:
    origin_access_identity_id: E17DRN9XUOAHZX
    caller_reference: this is an example reference
    comment: this is a new comment

- name: delete an existing origin access identity using caller_reference as an identifier
  community.aws.cloudfront_origin_access_identity:
    state: absent
    caller_reference: this is an example reference
    comment: this is a new comment
"""

RETURN = r"""
cloud_front_origin_access_identity:
  description: The origin access identity's information.
  returned: always
  type: complex
  contains:
    cloud_front_origin_access_identity_config:
      description: describes a url specifying the origin access identity.
      returned: always
      type: complex
      contains:
        caller_reference:
          description: a caller reference for the oai
          returned: always
          type: str
        comment:
          description: a comment describing the oai
          returned: always
          type: str
    id:
      description: a unique identifier of the oai
      returned: always
      type: str
    s3_canonical_user_id:
      description: the canonical user ID of the user who created the oai
      returned: always
      type: str
e_tag:
  description: The current version of the origin access identity created.
  returned: always
  type: str
location:
  description: The fully qualified URI of the new origin access identity just created.
  returned: when initially created
  type: str
"""

import datetime

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by imported AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.cloudfront_facts import CloudFrontFactsServiceManager

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class CloudFrontOriginAccessIdentityServiceManager(object):
    """
    Handles CloudFront origin access identity service calls to aws
    """

    def __init__(self, module):
        self.module = module
        self.client = module.client("cloudfront")

    def create_origin_access_identity(self, caller_reference, comment):
        try:
            return self.client.create_cloud_front_origin_access_identity(
                CloudFrontOriginAccessIdentityConfig={"CallerReference": caller_reference, "Comment": comment}
            )
        except (ClientError, BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Error creating cloud front origin access identity.")

    def delete_origin_access_identity(self, origin_access_identity_id, e_tag):
        try:
            result = self.client.delete_cloud_front_origin_access_identity(Id=origin_access_identity_id, IfMatch=e_tag)
            return result, True
        except (ClientError, BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Error deleting Origin Access Identity.")

    def update_origin_access_identity(self, caller_reference, comment, origin_access_identity_id, e_tag):
        changed = False
        new_config = {"CallerReference": caller_reference, "Comment": comment}

        try:
            current_config = self.client.get_cloud_front_origin_access_identity_config(Id=origin_access_identity_id)[
                "CloudFrontOriginAccessIdentityConfig"
            ]
        except (ClientError, BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Error getting Origin Access Identity config.")

        if new_config != current_config:
            changed = True

        try:
            # If the CallerReference is a value already sent in a previous identity request
            # the returned value is that of the original request
            result = self.client.update_cloud_front_origin_access_identity(
                CloudFrontOriginAccessIdentityConfig=new_config,
                Id=origin_access_identity_id,
                IfMatch=e_tag,
            )
        except (ClientError, BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Error updating Origin Access Identity.")

        return result, changed


class CloudFrontOriginAccessIdentityValidationManager(object):
    """
    Manages CloudFront Origin Access Identities
    """

    def __init__(self, module):
        self.module = module
        self.__cloudfront_facts_mgr = CloudFrontFactsServiceManager(module)

    def describe_origin_access_identity(self, origin_access_identity_id, fail_if_missing=True):
        try:
            return self.__cloudfront_facts_mgr.get_origin_access_identity(
                id=origin_access_identity_id, fail_if_error=False
            )
        except is_boto3_error_code("NoSuchCloudFrontOriginAccessIdentity") as e:  # pylint: disable=duplicate-except
            if fail_if_missing:
                self.module.fail_json_aws(e, msg="Error getting etag from origin_access_identity.")
            return {}
        except (ClientError, BotoCoreError) as e:  # pylint: disable=duplicate-except
            self.module.fail_json_aws(e, msg="Error getting etag from origin_access_identity.")

    def validate_etag_from_origin_access_identity_id(self, origin_access_identity_id, fail_if_missing):
        oai = self.describe_origin_access_identity(origin_access_identity_id, fail_if_missing)
        if oai is not None:
            return oai.get("ETag")

    def validate_origin_access_identity_id_from_caller_reference(self, caller_reference):
        origin_access_identities = self.__cloudfront_facts_mgr.list_origin_access_identities()
        origin_origin_access_identity_ids = [oai.get("Id") for oai in origin_access_identities]
        for origin_access_identity_id in origin_origin_access_identity_ids:
            oai_config = self.__cloudfront_facts_mgr.get_origin_access_identity_config(id=origin_access_identity_id)
            temp_caller_reference = oai_config.get("CloudFrontOriginAccessIdentityConfig").get("CallerReference")
            if temp_caller_reference == caller_reference:
                return origin_access_identity_id

    def validate_comment(self, comment):
        if comment is None:
            return "origin access identity created by Ansible with datetime " + datetime.datetime.now().strftime(
                "%Y-%m-%dT%H:%M:%S.%f"
            )
        return comment

    def validate_caller_reference_from_origin_access_identity_id(self, origin_access_identity_id, caller_reference):
        if caller_reference is None:
            if origin_access_identity_id is None:
                return datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f")
            oai = self.describe_origin_access_identity(origin_access_identity_id, fail_if_missing=True)
            origin_access_config = oai.get("CloudFrontOriginAccessIdentity", {}).get(
                "CloudFrontOriginAccessIdentityConfig", {}
            )
            return origin_access_config.get("CallerReference")
        return caller_reference


def main():
    argument_spec = dict(
        state=dict(choices=["present", "absent"], default="present"),
        origin_access_identity_id=dict(),
        caller_reference=dict(),
        comment=dict(),
    )

    result = {}
    e_tag = None
    changed = False

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=False)
    service_mgr = CloudFrontOriginAccessIdentityServiceManager(module)
    validation_mgr = CloudFrontOriginAccessIdentityValidationManager(module)

    state = module.params.get("state")
    caller_reference = module.params.get("caller_reference")

    comment = module.params.get("comment")
    origin_access_identity_id = module.params.get("origin_access_identity_id")

    if origin_access_identity_id is None and caller_reference is not None:
        origin_access_identity_id = validation_mgr.validate_origin_access_identity_id_from_caller_reference(
            caller_reference
        )

    if state == "present":
        comment = validation_mgr.validate_comment(comment)
        caller_reference = validation_mgr.validate_caller_reference_from_origin_access_identity_id(
            origin_access_identity_id, caller_reference
        )
        if origin_access_identity_id is not None:
            e_tag = validation_mgr.validate_etag_from_origin_access_identity_id(origin_access_identity_id, True)
            # update cloudfront origin access identity
            result, changed = service_mgr.update_origin_access_identity(
                caller_reference, comment, origin_access_identity_id, e_tag
            )
        else:
            # create cloudfront origin access identity
            result = service_mgr.create_origin_access_identity(caller_reference, comment)
            changed = True
    else:
        e_tag = validation_mgr.validate_etag_from_origin_access_identity_id(origin_access_identity_id, False)
        if e_tag:
            result, changed = service_mgr.delete_origin_access_identity(origin_access_identity_id, e_tag)

    result.pop("ResponseMetadata", None)

    module.exit_json(changed=changed, **camel_dict_to_snake_dict(result))


if __name__ == "__main__":
    main()



# Copyright (c) 2023-2025, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: lookup_rfc8427
author:
  - Felix Fontein (@felixfontein)
  - Vasiliy Kiryanov (@vasiliyk)
short_description: Look up DNS records and return RFC 8427 JSON format
version_added: 3.4.0
requirements:
  - dnspython >= 2.0.0
description:
  - Look up DNS records and return them in L(RFC 8427 DNS message JSON format, https://www.rfc-editor.org/rfc/rfc8427.html).
  - RFC 8427 defines a standardized format for representing DNS messages in JSON.
options:
  _terms:
    description:
      - Domain name(s) to query.
    type: list
    elements: str
    required: true
  type:
    description:
      - The record type to retrieve.
      - Support for V(HTTPS) and V(SVCB) has been added in community.dns 3.4.0.
    type: str
    default: A
    choices:
      - A
      - AAAA
      - CAA
      - CNAME
      - DNAME
      - DNSKEY
      - DS
      - HINFO
      - HTTPS
      - LOC
      - MX
      - NAPTR
      - NS
      - NSEC
      - NSEC3
      - NSEC3PARAM
      - PTR
      - RP
      - RRSIG
      - SOA
      - SPF
      - SRV
      - SSHFP
      - SVCB
      - TLSA
      - TXT
  query_retry:
    description:
      - Number of retries for DNS query timeouts.
    type: int
    default: 3
  query_timeout:
    description:
      - Timeout per DNS query in seconds.
    type: float
    default: 10
  server:
    description:
      - The DNS server(s) to use to look up the result. Must be a list of one or more IP addresses.
      - By default, the system's standard resolver is used.
    type: list
    elements: str
  servfail_retries:
    description:
      - How often to retry on SERVFAIL errors.
    type: int
    default: 0
  nxdomain_handling:
    description:
      - How to handle NXDOMAIN errors. These appear if an unknown domain name is queried.
      - V(empty) (default) returns an empty result for that domain name. This means that for the corresponding domain name,
        nothing is added to RV(_result).
      - V(fail) makes the lookup fail.
    type: str
    choices:
      - empty
      - fail
    default: empty
  search:
    description:
      - If V(false), the input is assumed to be an absolute domain name.
      - If V(true), the input is assumed to be a relative domain name if it does not end with C(.), the search list configured
        in the system's resolver configuration will be used for relative names, and the resolver's domain may be added to
        relative names.
    type: bool
    default: true
notes:
  - This plugin returns DNS messages in RFC 8427 JSON format, which includes C(Header), C(Question), C(Answer), C(Authority), and C(Additional) sections.
  - Note that when using this lookup plugin with V(lookup(\)), and the result is a one-element list, Ansible simply returns
    the one element not as a list. Since this behavior is surprising and can cause problems, it is better to use V(query(\))
    instead of V(lookup(\)). See the examples and also R(Forcing lookups to return lists, query) in the Ansible documentation.
"""

EXAMPLES = r"""
- name: Look up A (IPv4) records for example.org in RFC 8427 JSON format
  ansible.builtin.debug:
    msg: "{{ query('community.dns.lookup_rfc8427', 'example.org.') }}"

- name: Look up AAAA (IPv6) records for example.org in RFC 8427 JSON format
  ansible.builtin.debug:
    msg: "{{ query('community.dns.lookup_rfc8427', 'example.org.', type='AAAA' ) }}"

- name: Get complete DNS message for MX records
  ansible.builtin.debug:
    msg: "{{ query('community.dns.lookup_rfc8427', 'example.org.', type='MX' ) }}"
"""

RETURN = r"""
_result:
  description:
    - DNS messages in RFC 8427 JSON format for all queried DNS names.
    - Every element in O(_terms) corresponds to one element in this list.
  type: list
  elements: dict
  sample:
    - Header:
        ID: 12345
        QR: true
        Opcode: 0
        AA: false
        TC: false
        RD: true
        RA: true
        AD: false
        CD: false
        Rcode: 0
        QDCOUNT: 1
        ANCOUNT: 1
        NSCOUNT: 0
        ARCOUNT: 0
      Question:
        - name: example.org.
          type: 1
          class: 1
      Answer:
        - name: example.org.
          type: 1
          class: 1
          TTL: 3600
          data: 93.184.216.34
      Authority: []
      Additional: []
"""

import typing as t

from ansible.errors import AnsibleLookupError
from ansible.module_utils.common.text.converters import to_text
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.dns.plugins.module_utils._dnspython_records import (
    NAME_TO_RDTYPE,
    NAME_TO_REQUIRED_VERSION,
)
from ansible_collections.community.dns.plugins.module_utils._ips import is_ip_address
from ansible_collections.community.dns.plugins.module_utils._resolver import (
    SimpleResolver,
)
from ansible_collections.community.dns.plugins.plugin_utils._resolver import (
    assert_requirements_present as assert_requirements_present_dnspython,
)
from ansible_collections.community.dns.plugins.plugin_utils._resolver import (
    guarded_run,
)

try:
    import dns.flags
    import dns.message
    import dns.query
    import dns.rcode
    import dns.resolver
    from dns.rdatatype import RdataType
    from dns.resolver import NXDOMAIN
except ImportError:
    # handled by assert_requirements_present_dnspython
    pass


class LookupModule(LookupBase):
    @staticmethod
    def _convert_rrset_to_rfc8427(
        rrset: dns.rrset.RRset,
    ) -> list[dict[str, object]]:
        """Convert a DNS RRset to RFC 8427 format."""
        records = []
        for rdata in rrset:
            record = {
                "name": str(rrset.name).rstrip("."),  # RFC 8427: no trailing dot
                "type": int(rrset.rdtype),
                "class": dns.rdataclass.to_text(rrset.rdclass),
                "ttl": rrset.ttl,
                "data": str(rdata),
            }
            # For MX and similar, parse data as object if needed
            if rrset.rdtype == dns.rdatatype.MX:
                preference, exchange = str(rdata).split(" ", 1)
                record["data"] = {
                    "preference": int(preference),
                    "exchange": exchange.rstrip("."),
                }
            records.append(record)
        return records

    @staticmethod
    def _convert_message_to_rfc8427(
        message: dns.message.Message, question_name: str, question_type: int
    ) -> dict[str, object]:
        """Convert a DNS message to RFC 8427 JSON format."""

        def rrsets_to_records(rrsets: list[dns.rrset.RRset]) -> list[dict[str, object]]:
            records: list[dict[str, object]] = []
            for rrset in rrsets:
                records.extend(LookupModule._convert_rrset_to_rfc8427(rrset))
            return records

        # RFC 8427 header fields
        header = {
            "id": message.id,
            "flags": [
                f for f in dns.flags.to_text(message.flags).split() if f
            ],  # list of flag strings
            "rcode": dns.rcode.to_text(message.rcode()),
            "question_count": len(message.question),
            "answer_count": len(message.answer),
            "authority_count": len(message.authority),
            "additional_count": len(message.additional),
        }

        # RFC 8427 Question section
        question = [
            {
                "name": question_name.rstrip("."),
                "type": question_type,
                "class": dns.rdataclass.to_text(dns.rdataclass.IN),
            }
        ]

        result: dict[str, object] = {
            "Header": header,
            "Question": question,
            "Answer": rrsets_to_records(message.answer),
            "Authority": rrsets_to_records(message.authority),
            "Additional": rrsets_to_records(message.additional),
        }
        return result

    @staticmethod
    def _resolve(
        resolver: SimpleResolver,
        name: str,
        rdtype: RdataType,
        server_addresses: list[str] | None,
        nxdomain_handling: t.Literal["empty", "fail"],
        target_can_be_relative: bool = True,
        search: bool = True,
    ) -> dict[str, t.Any]:
        def callback() -> dict[str, t.Any]:
            try:
                rrset = resolver.resolve(
                    name,
                    rdtype=rdtype,
                    server_addresses=server_addresses,
                    nxdomain_is_empty=nxdomain_handling == "empty",
                    target_can_be_relative=target_can_be_relative,
                    search=search,
                )
                # Create a response message and append the answer
                query = dns.message.make_query(name, rdtype)
                response_msg = dns.message.make_response(query)
                if rrset:
                    response_msg.answer.append(rrset)
                return LookupModule._convert_message_to_rfc8427(
                    response_msg, name, int(rdtype)
                )
            except dns.resolver.NXDOMAIN:
                raise AnsibleLookupError(f"Got NXDOMAIN when querying {name}") from None

        return guarded_run(
            callback,
            error_class=AnsibleLookupError,
            server=name,
        )

    @staticmethod
    def _get_resolver(
        resolver: SimpleResolver, server: str
    ) -> t.Callable[[], list[str]]:
        def f():
            try:
                return resolver.resolve_addresses(server)
            except NXDOMAIN as exc:
                raise AnsibleLookupError(
                    f"Nameserver {server} does not exist ({exc})"
                ) from None

        return f

    def run(
        self, terms: list[t.Any], variables: t.Any | None = None, **kwargs: t.Any
    ) -> list[dict[str, t.Any]]:
        assert_requirements_present_dnspython("community.dns.lookup_rfc8427", "lookup")

        self.set_options(var_options=variables, direct=kwargs)

        resolver = SimpleResolver(
            timeout=self.get_option("query_timeout"),
            timeout_retries=self.get_option("query_retry"),
            servfail_retries=self.get_option("servfail_retries"),
        )

        record_type = self.get_option("type")
        if record_type not in NAME_TO_RDTYPE:
            min_version = NAME_TO_REQUIRED_VERSION[record_type]
            raise AnsibleLookupError(
                f"Your dnspython version does not support {record_type} records. You need version {min_version} or newer."
            )
        rdtype = NAME_TO_RDTYPE[record_type]

        nxdomain_handling: t.Literal["empty", "fail"] = self.get_option(
            "nxdomain_handling"
        )

        search: bool = self.get_option("search")

        server_addresses: list[str] | None = None
        if self.get_option("server"):
            server_addresses = []
            servers: list[str] = self.get_option("server")
            for server in servers:
                if is_ip_address(server):
                    server_addresses.append(server)
                    continue
                server_addresses.extend(
                    guarded_run(
                        self._get_resolver(resolver, server),
                        error_class=AnsibleLookupError,
                        server=server,
                    )
                )

        result = []
        for name in terms:
            result.append(
                self._resolve(
                    resolver,
                    to_text(name),
                    rdtype,
                    server_addresses,
                    nxdomain_handling,
                    target_can_be_relative=search,
                    search=search,
                )
            )
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Vasily Prokopov (@vasilyprokopov)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_interface_policy_link_level
short_description: Manage Link Level interface policies (fabric:HIfPol)
description:
- The link level interface policy specifies the layer 1 parameters of switch interfaces.
options:
  link_level_policy:
    description:
    - The name of the Link Level interface policy.
    type: str
    aliases: [ name ]
  description:
    description:
    - The description of the Link Level interface policy.
    type: str
    aliases: [ descr ]
  auto_negotiation:
    description:
    - Auto-negotiation enables devices to automatically exchange information over a link about speed and duplex abilities.
    - The APIC defaults to C(on) when unset during creation.
    type: bool
    default: true
  speed:
    description:
    - Determines the interface policy administrative port speed.
    - The C(auto) option is only supported in APIC version 5.2 or later.
    - The APIC defaults to C(inherit) when unset during creation.
    type: str
    choices: [ 100M, 1G, 10G, 25G, 40G, 50G, 100G, 200G, 400G, auto, inherit ]
    default: inherit
  link_debounce_interval:
    description:
    - Enables the debounce timer for physical interface ports and sets it for a specified amount of time in milliseconds.
    - The APIC defaults to C(100) when unset during creation.
    type: int
    default: 100
  forwarding_error_correction:
    description:
    - Determines the forwarding error correction (FEC) mode.
    - The APIC defaults to C(inherit) when unset during creation.
    type: str
    choices: [ inherit, kp-fec, cl91-rs-fec, cl74-fc-fec, disable-fec, ieee-rs-fec, cons16-rs-fec ]
    default: inherit
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fabric:HIfPol).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Vasily Prokopov (@vasilyprokopov)
"""

EXAMPLES = r"""
- name: Add a Link Level Policy
  cisco.aci.aci_interface_policy_link_level:
    host: apic
    username: admin
    password: SomeSecretPassword
    link_level_policy: link_level_policy_test
    description: via Ansible
    auto_negotiation: 'on'
    speed: 100M
    link_debounce_interval: 100
    forwarding_error_correction: cl91-rs-fec
    state: present
  delegate_to: localhost

- name: Remove a Link Level Policy
  cisco.aci.aci_interface_policy_link_level:
    host: apic
    username: admin
    password: SomeSecretPassword
    link_level_policy: ansible_test
    state: absent

- name: Query a Link Level Policy
  cisco.aci.aci_interface_policy_link_level:
    host: apic
    username: admin
    password: SomeSecretPassword
    link_level_policy: link_level_policy_test
    state: query
  delegate_to: localhost

- name: Query all Link Level Policies
  cisco.aci.aci_interface_policy_link_level:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        link_level_policy=dict(type="str", aliases=["name"]),
        description=dict(type="str", aliases=["descr"]),
        auto_negotiation=dict(type="bool", default="true"),
        speed=dict(type="str", default="inherit", choices=["100M", "1G", "10G", "25G", "40G", "50G", "100G", "200G", "400G", "auto", "inherit"]),
        link_debounce_interval=dict(type="int", default="100"),
        forwarding_error_correction=dict(
            type="str", default="inherit", choices=["inherit", "kp-fec", "cl91-rs-fec", "cl74-fc-fec", "disable-fec", "ieee-rs-fec", "cons16-rs-fec"]
        ),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["link_level_policy"]],
            ["state", "present", ["link_level_policy"]],
        ],
    )

    aci = ACIModule(module)

    link_level_policy = module.params["link_level_policy"]
    description = module.params["description"]
    auto_negotiation = aci.boolean(module.params["auto_negotiation"], "on", "off")
    speed = module.params["speed"]
    link_debounce_interval = module.params["link_debounce_interval"]
    if link_debounce_interval is not None and link_debounce_interval not in range(0, 5001):
        module.fail_json(msg='The "link_debounce_interval" must be a value between 0 and 5000')
    forwarding_error_correction = module.params["forwarding_error_correction"]
    state = module.params["state"]

    aci.construct_url(
        root_class=dict(
            aci_class="fabricHIfPol",
            aci_rn="infra/hintfpol-{0}".format(link_level_policy),
            module_object=link_level_policy,
            target_filter={"name": link_level_policy},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="fabricHIfPol",
            class_config=dict(
                name=link_level_policy,
                descr=description,
                autoNeg=auto_negotiation,
                speed=speed,
                linkDebounce=link_debounce_interval,
                fecMode=forwarding_error_correction,
            ),
        )

        aci.get_diff(aci_class="fabricHIfPol")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: s3_cors
version_added: 1.0.0
short_description: Manage CORS for S3 buckets in AWS
description:
  - Manage CORS for S3 buckets in AWS.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_s3_cors).
    The usage did not change.
author:
  - "Oyvind Saltvik (@fivethreeo)"
options:
  name:
    description:
      - Name of the S3 bucket.
    required: true
    type: str
  rules:
    description:
      - Cors rules to put on the S3 bucket.
    type: list
    elements: dict
  state:
    description:
      - Create or remove cors on the S3 bucket.
    required: true
    choices: [ 'present', 'absent' ]
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Create a simple cors for s3 bucket
- community.aws.s3_cors:
    name: mys3bucket
    state: present
    rules:
      - allowed_origins:
          - http://www.example.com/
        allowed_methods:
          - GET
          - POST
        allowed_headers:
          - Authorization
        expose_headers:
          - x-amz-server-side-encryption
          - x-amz-request-id
        max_age_seconds: 30000

# Remove cors for s3 bucket
- community.aws.s3_cors:
    name: mys3bucket
    state: absent
"""

RETURN = r"""
changed:
  description: check to see if a change was made to the rules
  returned: always
  type: bool
  sample: true
name:
  description: name of bucket
  returned: always
  type: str
  sample: 'bucket-name'
rules:
  description: list of current rules
  returned: always
  type: list
  sample: [
     {
        "allowed_headers": [
          "Authorization"
        ],
        "allowed_methods": [
          "GET"
        ],
        "allowed_origins": [
          "*"
        ],
        "max_age_seconds": 30000
      }
    ]
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.policy import compare_policies

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def create_or_update_bucket_cors(connection, module):
    name = module.params.get("name")
    rules = module.params.get("rules", [])
    changed = False

    try:
        current_camel_rules = connection.get_bucket_cors(Bucket=name)["CORSRules"]
    except ClientError:
        current_camel_rules = []

    new_camel_rules = snake_dict_to_camel_dict(rules, capitalize_first=True)
    # compare_policies() takes two dicts and makes them hashable for comparison
    if compare_policies(new_camel_rules, current_camel_rules):
        changed = True

    if changed:
        try:
            connection.put_bucket_cors(Bucket=name, CORSConfiguration={"CORSRules": new_camel_rules})
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg=f"Unable to update CORS for bucket {name}")

    module.exit_json(changed=changed, name=name, rules=rules)


def destroy_bucket_cors(connection, module):
    name = module.params.get("name")
    changed = False

    try:
        connection.delete_bucket_cors(Bucket=name)
        changed = True
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg=f"Unable to delete CORS for bucket {name}")

    module.exit_json(changed=changed)


def main():
    argument_spec = dict(
        name=dict(required=True, type="str"),
        rules=dict(type="list", elements="dict"),
        state=dict(type="str", choices=["present", "absent"], required=True),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec)

    client = module.client("s3")

    state = module.params.get("state")

    if state == "present":
        create_or_update_bucket_cors(client, module)
    elif state == "absent":
        destroy_bucket_cors(client, module)


if __name__ == "__main__":
    main()
